# Guía de usuario MSP430 GCC Toolchain

------

[^]: Para usuarios de Linux 

**Abstract**

This manual aims to describe the setup and basic operations of the MSP430 GCC toolchain to know how to use the open-source GCC compiler for microcontrollers to build an example for an MSP430 target device using the command terminal of Linux. It will also be shown how to debug software with the GDB Agent and the GNU Debugger tools so that the program that was built can be uploaded to the target device successfully.

------

##  Introducción

El paquete GCC de código abierto es un depurador completo y una cadena de herramientas de compilación C/C++ de código abierto para la creación y depuración de aplicaciones integradas basadas en microcontroladores MSP430. Este compilador soporta todos los dispositivos MSP430 sin limitaciones de tamaño de código y se puede utilizar de forma independiente desde la línea de comandos o dentro de Code Composer Studio v6.0 o posterior [1].

El paquete de instalación incluye lo siguiente:

1. Binarios GCC y GDB para Windows y Linux
2. Encabezado MSP430 y archivos de enlace
3. Código fuente
4. GBD Agent Configuration

#### Instalación de MSP430 GCC como paquete independiente

El primer paso es bastante sencillo, consiste en abrir la terminal de comando y direccionarse a la carpeta en donde se guardará el archivo que contenga el programa a ejecutar.
Con el comando **cd** se puede entrar a alguna carpeta en específico como lo puede ser la carpeta de **Descargas**, Ejemplo:

```c
$ cd Descargas
```

Antes de continuar con el siguiente paso se debe dirigir a la pagina oficial de TI (Texas Instruments) y yendo directamente a la siguiente pagina [https://www.ti.com/tool/MSP430\-GCC\-OPENSOURCE]() en la pestaña de descargas seleccione la opción mas factible para su computadora en este caso **Mitto Systems GCC 64-bit Linux installer incl. support files**.

Una vez descargada, se debe buscar en la carpeta de descargas el nombre del archivo.
Ya una vez se tiene el nombre del archivo descargado, en la terminal de comando se debe colocar lo siguiente, lo cual nos permitirá instalar el programa, colocando sudo **chmod +x <installer>.run** y el comando **./<installer>**. 

```c
$ sudo chmod +x msp430-gcc-full-linux-X64-installer-9.3.1.2.run
$ ./msp430-gcc-full-linux-X64-installer-9.3.1.2.run
```

Lo cual abrirá la siguiente ventana:

![](/home/griscampos/Descargas/msp430Guia/1.jpeg)

Seleccione el directorio de instalacion y de click en **Next** hasta finalizar .

Como se muestra en la siguiente imagen.

![](/home/griscampos/Descargas/msp430Guia/2.jpeg)

Crea una carpeta en donde colocaras el código.

> Comando para crear carpetas desde la terminal , el comando **mkdir** + el nombre del nuevo archivo.
> Comando para encontrar carpetas dentro de la carpeta en la que se encuentra **dir**.
> Comando para entrar en carpeta **cd**.

```c
$ mkdir Programa1
$ dir 
$ cd Programa1     
```

Ya dentro de la carpeta en la que colocaras el código coloca el siguiente comando para colocar el código desde la terminal.

```c
$ main.c
```

En el cual se copiara y pegara cualquier código a utilizar.
Ejemplo de código para hacer parpadear un led:

```c
int main(void) {
  volatile int i;
 
  // stop watchdog timer\\
 WDTCTL = WDTPW | WDTHOLD;
  // set up bit 0 of P1 as output\\
  P1DIR = 0x01;
  // intialize bit 0 of P1 to 0\\
  P1OUT = 0x00;
 
  // loop forever\\
  for (;;) {
    // toggle bit 0 of P1\\
    %P1OUT ^= 0x01;
    // delay for a while\\
    for (i = 0; i < 0x6000; i++);
  }
}
```

#### Compilar un programa desde la termial de Linux

El programa se puede compilar manualmente desde la terminal indicando las rutas de instalación de las librerías, binarios y demás, o usando un método más sencillo implementando un archivo *Makefile* incluido en los ejemplos por defecto que el paquete contiene. En este documento se muestran los pasos a seguir empleando el segundo método comenzando con la creación de un nuevo proyecto; sin embargo, a continuación, se muestra un ejemplo de compilación manual desde la terminal.

Ejemplo de compilación de un programa desde la terminal (primer método):

```c
$ ./msp430-elf-gcc  -I ~/msp430-gcc/include -L ~/msp430-gcc/include -T ~/msp430-gcc/include/msp430fr6989.ld -mmcu=msp430fr6989 -O2 -g ~/Documentos/Code/main.c -o ~/Documentos/Code/main.o
```

Primero se debe dirigir a la dirección en donde se encuentra instalado el paquete *MSP430- GCC* y redireccionarse a la carpeta *bin*.
Para compilar se coloca ./ seguido de la dirección *msp430-elf-gcc*. 
Y después coloca los parámetros correspondientes a -I, -L, -T, -g, -o, mmcu
Donde:
-I :Agrega el directorio a la lista de directorios para buscar archivos de encabezado.
-L: Agrega rutas en las que ld buscara bibliotecas de archivo.
-T: Dirección donde se encuentra las librerías del microcontrolador\\ especificando cual utilizaras 
-g: dirección donde se encuentra tu archivo .c
-o: dirección donde se encuentra tu archivo .c con la diferencia que se cambia el .c por .o
Mmcu: especifica el microcontrolador que utilizaras 

## **Creación de un nuevo proyecto** 

------

##### Pasos para la creación de un nuevo proyecto

1. Crear un directorio para el nuevo proyecto.
2. Insertar el archivo .c que contiene el programa que se subirá a la tarjeta. Para crear un archivo .c vea el apartado **Creación de un archivo .c**.
3. Dentro de la carpeta que se instaló (msp430-gcc), hay un directorio llamado **examples** que contiene ejemplos predeterminados. Copiar uno de los archivos **Makefile** de los proyectos de ejemplo dentro del directorio de su nuevo proyecto.
4. Abrir el archivo **Makefile** que copió y cambiar el formato de estructura actual por el que se muestra en la Figura Estructura de archivo Makefile. En la variable **OBJECTS** se debe colocar el nombre del programa que se va a subir (archivo .c) respetando la terminación **.o**. En la variable **DEVICE** se debe colocar el nombre del microcontrolador que se usará.
5. Colocar las rutas de instalación correpondientes para las variables **GCC\_DIR** y **SUPPORT\_FILE\_DIRECTORY**. Las direcciones para las variables son **../../../bin** y **../../../include** respectivamente como se muestra en el siguiente ejemplo.

```c
OBJECTS=main.o

GCC_DIR = /home/usuario/msp430-gcc/bin
SUPPORT_FILE_DIRECTORY = /home/usuario/msp430-gcc/include

DEVICE  = msp430fr6989
CC      = $(GCC_DIR)/msp430-elf-gcc
GDB     = $(GCC_DIR)/msp430-elf-gdb

CFLAGS = -I $(SUPPORT_FILE_DIRECTORY) -mmcu=$(DEVICE) -O2 -g
LFLAGS = -L $(SUPPORT_FILE_DIRECTORY) -T $(DEVICE).ld

all: ${OBJECTS}
	$(CC) $(CFLAGS) $(LFLAGS) $? -o $(DEVICE).out


debug: all
	$(GDB) $(DEVICE).out
```

#### Creación de un archivo .c

Para crear un archivo .c dirijase en el directorio en el que lo desea crear y digite el siguiente comando **pico filename.c**. Por fines explicativos, como ejemplo, se creará un archivo .c llamado **Ejemplo** en la carpeta **Programa1**,como se muestra el comando antes mencionado adaptado al ejemplo.

```c
$ pico ejemplo.c
```

Para crear un archivo .c dirijase en el directorio en el que lo desea crear y digite el siguiente comando **pico filename.c**. Por fines explicativos, como ejemplo, se creará un archivo .c llamado **Ejemplo** en la carpeta **Programa1**, en la Figura se muestra el comando antes mencionado adaptado al ejemplo.

```c
#include <msp430fr6989.h>
 
int main(void) {
  volatile int i;
 
  // stop watchdog timer
 WDTCTL = WDTPW | WDTHOLD;
  // set up bit 0 of P1 as output
  P1DIR = 0x01;
  // intialize bit 0 of P1 to 0
  P1OUT = 0x00;
 
  // loop forever
  for (;;) {
    // toggle bit 0 of P1
    P1OUT ^= 0x01;
    // delay for a while
    for (i = 0; i < 0x6000; i++);
    

[ 26 líneas leídas ]
^G Ayuda         ^O Guardar     ^W Buscar   ^K Cortar        ^T Ejecutar      ^C Ubicación     M-U Deshacer     M-A Poner marca  M-] A llave      M-Q Anterior   ^B Atrás   ^◂ Palabr ant
^X Salir         ^R Leer fich.    ^\ Reemplazar    ^U Pegar      ^J Justificar   ^/ Ir a línea M-E Rehacer  M-6 Copiar  ^Q Buscar atrás  M-W Siguiente    ^F Adelante   ^▸ Palabra Siguiente

```

## **Debugging**

#### **Uso del agente GDB **

###### Introducción 

El agente GDB es una herramienta para conectar el GDB con el hardware de destino para depurar su software. El GDB
El agente utiliza la pila de depuración MSP430 para conectarse al hardware y proporciona una interfaz al GDB.
En Windows, se proporciona una consola y una versión de aplicación GUI del agente GDB. Solo la aplicación de consola es compatible con Linux.

#### Iniciando GDB Agent

###### Línea de comando

Abra una terminal de comando y dirijase al directorio de instalación, una vez que lo haga, ejecute lo siguiente:

```c
$ ./bin/gdb\_agent\_console msp430.dat
```

La aplicación de consola abre un puerto TCP/IP en el equipo local. Muestra el número de puerto en la consola. Por defecto, este número de puerto es 55000.

[^]: Es importante que deje abierta esta consola hasta terminar de depurar el programa (debugging) para poder conectar el GDB con el GDB Agent.

```c
$ ./bin/gdb\_agent\_console msp430.dat
CPU Name             Port
--------             ----
msp430              :55000

Starting all cores
CPU Name             Status
--------             ------
msp430               Waiting for client
```

#### Debugging con GDB

##### Ejecutar un programa en el debugger

1. Abra otra terminal de comando y dirijase a la ruta donde se encuentra el directorio con el archivo **.c** y el Makefile del programa que quiere cargar a la tarjeta.

2. Escriba el comando **make debug**. Este comando inicia el GDB y espera que se introduzcan comandos, esto es indicado por el puntero **(gdb)**. 

   ```c
   $ make debug
   /home/usuario/msp430-gcc/bin/msp430-elf-gcc -I /home/usuario/msp430-gcc/include -mmcu=msp430fr6989 -O2 -g -L /home/usuario/msp430-gcc/include -T msp430fr6989.ld  Ejemplo.o -o msp430fr6989.out
   /home/usuario/msp430-gcc/bin/msp430-elf-gdb msp430fr6989.out
   GNU gdb (Mitto Systems Limited - msp430-gcc 9.3.1.11) 9.1
   Copyright (C) 2020 Free Software Foundation, Inc.
   License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>
   This is free software: you are free to change and redistribute it.
   There is NO WARRANTY, to the extent permitted by law.
   Type "show copying" and "show warranty" for details.
   This GDB was configured as "--host=x86_64-pc-linux-gnu --target=msp430-elf".
   Type "show configuration" for configuration details.
   For bug reporting instructions, please see:
   <http://www.gnu.org/software/gdb/bugs/>.
   Find the GDB manual and other documentation resources online at:
       <http://www.gnu.org/software/gdb/documentation/>.
   
   For help, type "help".
   Type "apropos word" to search for commands related to "word"...
   Reading symbols from msp430fr6989.out...
   (gdb) 
   ```

3. Conecte el dispositivo.

4. Escriba el comando **target remote :55000** y presione enter para conectar el GDB con el GDB Agent que se encuentra en la otra terminal de comando que se dejó abierta. En la otra terminal se podrá observar en la consola del GDB Agent que se conectó un dispositivo/cliente.

   ```c
   Reading symbols from msp430fr6989.out...
   (gdb) target remote :55000
   Remote debugging using :55000
   0x00004402 in __crt0_start ()
   (gdb) 
   ```

   Consola del GDB Agent:

   ```c
   $ ./bin/gdb\_agent\_console msp430.dat
   CPU Name             Port
   --------             ----
   msp430              :55000
   
   Starting all cores
   CPU Name             Status
   --------             ------
   msp430               Waiting for client
   msp430               Client connected...Connecting to Target
   Found USB FET at ttyACM0
   Target connected...Starting server
   ```

5. Escriba **load** y presione enter para cargar el programa binario al dispositivo MSP430 que esté usando.

   ```c
   (gdb) load
   Loading section .upper.data, size 0x2 lma 0x4400
   Loading section .text, size 0x3c lma 0x4402
   Loading section __reset_vector, size 0x2 lma 0xfffe
   Start address 0x00004402, load size 64
   Transfer rate: 71 bytes/sec, 21 bytes/write.
   (gdb)
   ```

   

6. Escriba el comando **continue** para correr el programa o en su defecto escriba **step** para compilar elprograma por partes.

   ```c
   (gdb) continue
   Continuing.
   ```

   

7. Para parar la ejecución teclee **Ctrl + C** (^C), y para salir del GDB escriba **q**, presione enter y posteriormente escriba **y** y presione enter.

   ```c
   ^C
   Program stopped.
   main () at Ejemplo.c:15
   15		        i--;
   (gdb) q
   A debugging session is active.
   
   	Inferior 1 [Remote target] will be detached.
   
   Quit anyway? (y or n) y
   Detaching from program: /home/usuario/Escritorio/msp430_programs/Programa1/msp430fr6989.out, Remote target
   Ending remote debugging.
   [Inferior 1 (Remote target) detached]
   ```

### **Solución de problemas**

[^]: En este apartado se muestran las posibles soluciones a los problemas más comunes.

**Error Make debug*

```c
$ make debug
Make: *** There is no rule to build the 'debug' target. Tall
```

Instala make para solucionar el problema mediante el comando :

```c
sudo apt-get install make
```

------

 **Error 127 makefile debug*

```c
/home/usuario/msp430-gcc/bin/msp430-elf-gdb:error while loading shared libraries: libncursesw.so.5: cannot open shared object file: No such file or directory 
make: *** [Makefile: debug] Error 127
```

Instala la libreria libncursesw.so.5 para solucionar el problema mediante el comando :

```c
sudo apt-get install libncursesw.so.5
```

------

